/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.directorysize;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;

import com.carlosbcruz.util.FileSupport;

/**
 * Inform the disk space used by each subdirectory of a given directory.
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class DirectoriesSize {

    private static class DirectoryInformation implements Comparable {

        private String directoryName = new String();
        private long size = 0L;
        private int files = 0;

        /**
         * Construct a directory information bean
         * @param directoryName The name of the directory
         * @param size the sum of sizes of all files
         * @param files the number of files
         */
        public DirectoryInformation(String directoryName, long size, int files) {

            super();
            this.directoryName = directoryName;
            this.size = size;
            this.files = files;
        }

        /**
         * Inform the name of the directory
         * @return the directoryName
         */
        String getDirectoryName() {

            return directoryName;
        }

        /**
         * Inform the sum of sizes of all files
         * @return the files
         */
        int getFiles() {

            return files;
        }

        /**
         * Inform the number of files
         * @return the size
         */
        long getSize() {

            return size;
        }

        /* (non-Javadoc)
         * @see java.lang.Comparable#compareTo(T)
         */
        public int compareTo(Object arg0) {

            DirectoryInformation directory = (DirectoryInformation)arg0;

            if (size < directory.getSize())
                return 1;
            else
                if (size == directory.getSize())
                    return 0;
                else
                    return -1;
        }

    }

    /**
     * Starting point of execution
     * @param args
     */
    public static void main(String[] args) {

        if (args.length != 1) {
            System.out.println("Usage java DirectoriesSize targetDirectory");
            return;
        }

        File file = new File(args[0]);

        if (!file.isDirectory()) {
            System.out.println("The first parameter should be a directory: "
                + args[0]);
            return;
        }

        ArrayList directoriesInformation = new ArrayList();

        String directoryList[] = file.list();
        for (int i = 0; i < directoryList.length; i++) {

            System.out
                    .println((i + 1) + " of " + directoryList.length + " ok.");

            File internalDirectory = new File(args[0] + "/" + directoryList[i]);
            if (internalDirectory.isDirectory()) {
                String directoryFiles[] = FileSupport.getFileOnSubfolder(args[0] + "/"
                    + directoryList[i], null);

                long size = 0L;
                int files = 0;
                for (int j = 0; j < directoryFiles.length; j++) {

                    File internalFile = new File(directoryFiles[j]);
                    if (internalFile.isFile()) {
                        size += internalFile.length();
                        files++;
                    }

                }

                DirectoryInformation information = new DirectoryInformation(
                    directoryList[i], size, files);
                directoriesInformation.add(information);
            }
        }

        Collections.sort(directoriesInformation);

        for (int i = 0; i < directoriesInformation.size(); i++) {
            DirectoryInformation information = (DirectoryInformation)directoriesInformation
                    .get(i);

            System.out.print(information.getDirectoryName() + " - ");

            if (information.getSize() < (1024L * 1024L)) {
                System.out.println("Files: " + information.getFiles()
                    + " Size: " + (information.getSize() / 1024L) + "Kb");
            } else {
                System.out.println("Files: " + information.getFiles()
                    + " Size: " + (information.getSize() / (1024L * 1024L))
                    + "Mb");
            }

        }
    }
}
