/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.multithread;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;

/**
 * The control painel.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class ControlPainel extends JFrame {

	private static final long serialVersionUID = 1L;

	javax.swing.Timer timer;

	private static Calendar calendar;
	private java.util.Date date;
	private long startSecond;

	private MultithreadManager manager;
	private boolean started = false;

	private Vector sources = new Vector();
	private Vector clients = new Vector();

	private static final int MAX_THREADS = 100;
	Thread allThreads[] = new Thread[MAX_THREADS];
	int threadIndex = 0;

	JLabel numberOfTasks = new JLabel();
	JLabel clientsNumber = new JLabel();
	GridLayout gridLayout1 = new GridLayout();
	JLabel sourcesNumber = new JLabel();
	JButton addClient = new JButton();
	JButton removeClient = new JButton();
	JButton startButton = new JButton();
	JButton endButton = new JButton();
	JButton addSource = new JButton();
	JButton removeSource = new JButton();
	JLabel secondsLabel = new JLabel();
	JLabel tasksRetrievedLabel = new JLabel();
	JLabel jLabel1 = new JLabel();

	public ControlPainel() {
		try {
			jbInit();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void jbInit() throws Exception {

		numberOfTasks.setText("No tasks");
		this.getContentPane().setLayout(gridLayout1);
		clientsNumber.setText("No clients");
		gridLayout1.setRows(6);
		gridLayout1.setColumns(2);
		sourcesNumber.setText("No sources");
		addClient.setEnabled(false);
		addClient.setText("Add Client");
		addClient.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addClient_actionPerformed(e);
			}
		});
		removeClient.setEnabled(false);
		removeClient.setToolTipText("");
		removeClient.setText("Remove Client");
		removeClient.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeClient_actionPerformed(e);
			}
		});
		startButton.setText("Start");
		startButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				startButton_actionPerformed(e);
			}
		});
		endButton.setEnabled(false);
		endButton.setText("Stop");
		endButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				endButton_actionPerformed(e);
			}
		});
		addSource.setEnabled(false);
		addSource.setText("Add Source");
		addSource.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addSource_actionPerformed(e);
			}
		});
		removeSource.setEnabled(false);
		removeSource.setToolTipText("");
		removeSource.setText("Remove Source");
		removeSource.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeSource_actionPerformed(e);
			}
		});
		secondsLabel.setText("0 seconds");
		tasksRetrievedLabel.setText("No tasks retrieved");
		this.getContentPane().add(numberOfTasks, null);
		this.getContentPane().add(sourcesNumber, null);
		this.getContentPane().add(clientsNumber, null);
		this.getContentPane().add(secondsLabel, null);
		this.getContentPane().add(tasksRetrievedLabel, null);
		this.getContentPane().add(startButton, null);
		this.getContentPane().add(endButton, null);
		this.getContentPane().add(addSource, null);
		this.getContentPane().add(removeSource, null);
		this.getContentPane().add(addClient, null);
		this.getContentPane().add(removeClient, null);
		this.getContentPane().add(jLabel1, null);
	}

	void removeClient_actionPerformed(ActionEvent e) {
		if (!started)
			return;
		if (clients.size() == 0)
			return;

		Client client = (Client) clients.remove(0);
		client.stop();
	}

	void startButton_actionPerformed(ActionEvent e) {
		if (started)
			return;
		started = true;
		if (manager == null)
			manager = new TasksRepository();
		startButton.setEnabled(false);
		endButton.setEnabled(true);
		addSource.setEnabled(true);
		removeSource.setEnabled(true);
		addClient.setEnabled(true);
		removeClient.setEnabled(true);

		calendar = Calendar.getInstance();
		date = calendar.getTime();
		startSecond = date.getTime();

		timer = new javax.swing.Timer(100, new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				numberOfTasks.setText(manager.getNumberOfTasks() + " tasks");
				sourcesNumber.setText(sources.size() + " sources");
				clientsNumber.setText(clients.size() + " clients");
				tasksRetrievedLabel.setText(manager.getTasksRetrieved()
						+ " retrieved tasks");
				calendar = Calendar.getInstance();
				date = calendar.getTime();
				secondsLabel.setText((date.getTime() - startSecond) / 1000
						+ " seconds");

			}
		});
		timer.start();
	}

	public static void main(String a[]) {
		ControlPainel controlPanel = new ControlPainel();
		controlPanel.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		controlPanel.pack();
		controlPanel.setVisible(true);
	}

	void endButton_actionPerformed(ActionEvent e) {
		if (!started)
			return;

		timer.stop();
		timer = null;

		started = false;
		startButton.setEnabled(true);
		endButton.setEnabled(false);
		addSource.setEnabled(false);
		removeSource.setEnabled(false);
		addClient.setEnabled(false);
		removeClient.setEnabled(false);

		Enumeration enumn = sources.elements();
		while (enumn.hasMoreElements()) {
			Sourcer source = (Sourcer) enumn.nextElement();
			source.stop();
		}
		sources = new Vector();

		enumn = clients.elements();
		while (enumn.hasMoreElements()) {
			Client client = (Client) enumn.nextElement();
			client.stop();
		}
		clients = new Vector();

		threadIndex = 0;
		numberOfTasks.setText("No tasks");
		clientsNumber.setText("No clients");
		sourcesNumber.setText("No sources");
		tasksRetrievedLabel.setText("No tasks retrieved");
		numberOfTasks.setText(manager.getNumberOfTasks() + " tasks");

	}

	void addSource_actionPerformed(ActionEvent e) {
		if (!started)
			return;
		if (threadIndex == MAX_THREADS)
			return;
		Sourcer sourcer = new Sourcer(manager);

		Thread thread = new Thread(sourcer);
		allThreads[threadIndex++] = thread;
		sources.add(sourcer);
		thread.start();
	}

	void addClient_actionPerformed(ActionEvent e) {
		if (!started)
			return;
		if (threadIndex == MAX_THREADS)
			return;

		Client client = new Client(manager);
		Thread thread = new Thread(client);
		allThreads[threadIndex++] = thread;
		clients.add(client);
		thread.start();

	}

	void removeSource_actionPerformed(ActionEvent e) {
		if (!started)
			return;
		if (sources.size() == 0)
			return;

		Sourcer source = (Sourcer) sources.remove(0);
		source.stop();

	}
}