/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.pageschanges;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * Control the properties.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class PropertiesController {

	private static Properties properties = new Properties();

	private static String PROPERTIES_FILE_NAME = "Default.properties";

	private static String PROPERTIES_FILE_HEADER = "Properties";

	static {

		File inputPropertiesFile = new File(PROPERTIES_FILE_NAME);
		FileInputStream inputStream = null;
		try {
			inputStream = new FileInputStream(inputPropertiesFile);
			try {
				properties.load(inputStream);

			} catch (IOException e1) {

				e1.printStackTrace();
			}
		} catch (FileNotFoundException e) {

// do nothing.File may be created
		}

	}

	/**
	 * Get the property to a key. A default value is returned if not found
	 * 
	 * @param propertyKey
	 *            The parameter key
	 * @return the value
	 */
	public static String getStringProperty(String propertyKey) {

		String property = properties.getProperty(propertyKey);

		return property;
	}

	public static void removeProperty(String propertyKey) {
		properties.remove(propertyKey);
	}
	
	/**
	 * Set an Account property
	 * 
	 * @param propertyKey
	 *            the Key
	 * @param propertyValue
	 *            the Value
	 */
	public static void setStringProperty(String propertyKey,
			String propertyValue) {

		properties.setProperty(propertyKey, propertyValue);
	}

	/**
	 * Save the properties in the application file
	 */
	public static void saveProperties() {

		File outputPropertiesFile = new File(PROPERTIES_FILE_NAME);

		try {

			outputPropertiesFile.createNewFile();

		} catch (IOException e1) {

			e1.printStackTrace();
			return;
		}

		FileOutputStream outputStream = null;
		try {

			outputStream = new FileOutputStream(outputPropertiesFile);

		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		}

		try {

			properties.store(outputStream, PROPERTIES_FILE_HEADER);

		} catch (IOException e2) {

			e2.printStackTrace();
			return;
		}

	}
}
