/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */
package com.carlosbcruz.percurse.io;

import java.util.ArrayList;
import java.util.StringTokenizer;

import com.carlosbcruz.percurse.beans.FuelBean;
import com.carlosbcruz.percurse.messages.MessageCentralizer;
import com.carlosbcruz.percurse.util.CommaSeparetedException;
import com.carlosbcruz.percurse.util.CommaSeparetedValuesCallbackObject;

/**
 * Reads the fuel information
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class FuelReaderCallBack implements CommaSeparetedValuesCallbackObject {

	// Content of each column
	private final int DATE = 1, VOLUME = 2, ODOMETER = 3, COST = 4;

	// Store all fuels
	ArrayList fuels = new ArrayList();

	// Fuel Bean being created
	FuelBean fuelBean;

	// Validate format
	int validateMonth = 0;

	// Validate format
	int validateDay = 0;

	// Validate format
	int validateYear = 0;

	// Validate format
	float validateVolume = 0;

	// Validate format
	long validateOdometer = 0;

	// Validate format
	float validateCost = 0;

	// Help validate the incremental nature of odometers
	long previousOdometer = 0;

	/**
	 * Return the fuels
	 * 
	 * @return Returns the fuels.
	 */
	public FuelBean[] getFuels() {

		FuelBean[] fuelsArray = new FuelBean[fuels.size()];
		return (FuelBean[]) fuels.toArray(fuelsArray);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.carlosbcruz.percurse.util.CommaSeparetedValuesCallbackObject#newLineFound
	 * ()
	 */
	public void newLineFound() {

		if (validateMonth != 0 && validateDay != 0 && validateYear != 0
				&& validateVolume != 0 && validateOdometer != 0
				&& validateCost != 0) {

			fuelBean = new FuelBean(validateDay, validateMonth, validateYear,
					validateVolume, validateCost, validateOdometer);
			fuels.add(fuelBean);

			validateMonth = 0;
			validateDay = 0;
			validateYear = 0;
			validateVolume = 0;
			validateOdometer = 0;
			validateCost = 0;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.carlosbcruz.percurse.util.CommaSeparetedValuesCallbackObject#elementFound
	 * (int, int, java.lang.String)
	 */
	public void elementFound(int line, int column, String element)
			throws CommaSeparetedException {

		switch (column) {

		case (DATE): {

			String day, month, year;
			StringTokenizer tokenizer = new StringTokenizer(element, "/");

			// Validate token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			// Get token
			month = tokenizer.nextToken();

			try {
				validateMonth = Integer.parseInt(month);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateMonth < 1 || validateMonth > 12)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			// Validate next token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);

			// Get token
			day = tokenizer.nextToken();

			try {
				validateDay = Integer.parseInt(day);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateDay < 1 || validateDay > 31)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			// Validate token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			// Get token
			year = tokenizer.nextToken();

			try {
				validateYear = Integer.parseInt(year);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateYear < 2000 || validateYear > 2100)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			break;
		}

		case (VOLUME): {

			try {
				validateVolume = Float.parseFloat(element);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}

			break;

		}

		case (ODOMETER): {

			try {
				validateOdometer = Long.parseLong(element);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}

			if (validateOdometer < previousOdometer)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			previousOdometer = validateOdometer;

			break;

		}

		case (COST): {

			try {
				validateCost = Float.parseFloat(element);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}

			break;

		}

		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seecom.carlosbcruz.percurse.util.CommaSeparetedValuesCallbackObject#
	 * noMoreElementsFound()
	 */
	public void noMoreElementsFound() {
		// do nothing
	}

}
