/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */
package com.carlosbcruz.percurse.io;

import java.util.ArrayList;
import java.util.StringTokenizer;

import com.carlosbcruz.percurse.beans.PercurseBean;
import com.carlosbcruz.percurse.messages.MessageCentralizer;
import com.carlosbcruz.percurse.util.CommaSeparetedException;
import com.carlosbcruz.percurse.util.CommaSeparetedValuesCallbackObject;

/**
 * Controls the creation of com.carlosbcruz.percurse beans
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class PercurseReaderCallBack implements
		CommaSeparetedValuesCallbackObject {

	// Content of each column
	private final int DATE = 1, START_HOUR = 2, END_HOUR = 3, CATEGORY = 4,
			ODOMETER = 5;

	// Store all percurses
	ArrayList percurses = new ArrayList();

	// Percurse being created
	PercurseBean percurseBean;

	// Help validate the incremental nature of odometers
	long previousOdometer = 0;

	// Validate format
	int validateMonth = 0;

	// Validate format
	int validateDay = 0;

	// Validate format
	int validateYear = 0;

	// Validate format
	int validateStartHour = -1;

	// Validate format
	int validateStartMinute = -1;

	// Validate format
	int validateEndHour = -1;

	// Validate format
	int validateEndMinute = -1;

	// Validate category
	String validateCategory = null;

	// Validate format
	long validateOdometer = 0;

	/**
	 * Return the fuels
	 * 
	 * @return Returns the fuels.
	 */
	public PercurseBean[] getPercurses() {
		PercurseBean[] percurseArray = new PercurseBean[percurses.size()];
		return (PercurseBean[]) percurses.toArray(percurseArray);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.carlosbcruz.percurse.util.CommaSeparetedValuesCallbackObject#newLineFound()
	 */
	public void newLineFound() {

		if (validateMonth != 0 && validateDay != 0 && validateYear != 0
				&& validateStartHour != -1 && validateStartMinute != -1
				&& validateEndHour != -1 && validateEndMinute != -1
				&& validateCategory != null && validateOdometer != 0) {
			percurseBean = new PercurseBean(validateDay, validateMonth,
					validateYear, validateStartHour, validateStartMinute,
					validateEndHour, validateEndMinute, validateCategory,
					validateOdometer);
			percurses.add(percurseBean);
		}
		
		validateDay = 0;
		validateMonth = 0;
		validateYear = 0;
		validateStartHour = -1;
		validateStartMinute = -1;
		validateEndHour = -1;
		validateEndMinute = -1;
		validateCategory = null;
		validateOdometer = 0;
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.carlosbcruz.percurse.util.CommaSeparetedValuesCallbackObject#elementFound(int,
	 *      int, java.lang.String)
	 */
	public void elementFound(int line, int column, String element)
			throws CommaSeparetedException {
		switch (column) {

		case (DATE): {
			String day, month, year;
			StringTokenizer tokenizer = new StringTokenizer(element, "/");

			// Validate token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			// Get token
			month = tokenizer.nextToken();

			try {
				validateMonth = Integer.parseInt(month);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateMonth < 1 || validateMonth > 12)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			// Validate next token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			// Get token
			day = tokenizer.nextToken();

			try {
				validateDay = Integer.parseInt(day);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateDay < 1 || validateDay > 31)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			// Validate token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			// Get token
			year = tokenizer.nextToken();

			try {
				validateYear = Integer.parseInt(year);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateYear < 2000 || validateYear > 2100)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			break;
		}

		case (START_HOUR): {
			String hour, minute;
			StringTokenizer tokenizer = new StringTokenizer(element, ":");

			// Validate token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			// Get token
			hour = tokenizer.nextToken();

			try {
				validateStartHour = Integer.parseInt(hour);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateStartHour < 0 || validateStartHour > 23)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			// Validate token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			// Get token
			minute = tokenizer.nextToken();

			try {
				validateStartMinute = Integer.parseInt(minute);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateStartMinute < 0 || validateStartMinute > 59)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			break;
		}

		case (END_HOUR): {
			String hour, minute;
			StringTokenizer tokenizer = new StringTokenizer(element, ":");

			// Validate token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			// Get token
			hour = tokenizer.nextToken();
			try {
				validateEndHour = Integer.parseInt(hour);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateEndHour < 0 || validateEndHour > 23)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			// Validate token existance
			if (!tokenizer.hasMoreElements())
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			// Get token
			minute = tokenizer.nextToken();

			try {
				validateEndMinute = Integer.parseInt(minute);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}
			// Validate content
			if (validateEndMinute < 0 || validateEndMinute > 59)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			break;
		}
		case (CATEGORY): {

			if (element == null || element.equals(""))
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			validateCategory = new String(element);

			break;
		}

		case (ODOMETER): {

			try {
				validateOdometer = Long.parseLong(element);
			} catch (NumberFormatException e) {
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_DATA_FORMAT),
						line, column, element);
			}

			if (validateOdometer < previousOdometer)
				throw new CommaSeparetedException(MessageCentralizer
						.getMessage(MessageCentralizer.INVALID_CONTENT), line,
						column, element);

			previousOdometer = validateOdometer;

			break;
		}

		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.carlosbcruz.percurse.util.CommaSeparetedValuesCallbackObject#noMoreElementsFound()
	 */
	public void noMoreElementsFound() {
		// do nothing
	}

}
