/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */
package com.carlosbcruz.percurse.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.StringTokenizer;

/**
 * Reader for comma separeted values file.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class CommaSeparetedValuesFileReader {

	private String fileName;

	private CommaSeparetedValuesCallbackObject callBackObject;

	/**
	 * Creates a csv type reader and calling the methods of callback object for
	 * each tokens found
	 * 
	 * @param callBackObject
	 */
	public CommaSeparetedValuesFileReader(String fileName,
			CommaSeparetedValuesCallbackObject callBackObject) {
		this.fileName = fileName;
		this.callBackObject = callBackObject;
	}

	/**
	 * Reads the file and make the calls to the callback methods.
	 */
	public void read() throws FileNotFoundException, IOException, CommaSeparetedException {

		File inputFile = new File(fileName);
		FileReader fileReader = new FileReader(inputFile);
		LineNumberReader bufferedLineNumberReader = new LineNumberReader(
				fileReader);

		// For each line of the file
		String line = null;
		while ((line = bufferedLineNumberReader.readLine()) != null) {
			// Report the existance of a new line
			callBackObject.newLineFound();
			
			// Report each token
			StringTokenizer tokenizer = new StringTokenizer(line, CommaSeparetedValuesConstants.getMessage(CommaSeparetedValuesConstants.COMMA_SEPARETED_TOKEN));
			int currentColumn = 0;
			while (tokenizer.hasMoreElements()) {
				String nextToken = tokenizer.nextToken();
				callBackObject.elementFound(bufferedLineNumberReader.getLineNumber(), ++currentColumn, nextToken);
			}
		}
		// Report that there is nothing more to do
		callBackObject.noMoreElementsFound();
	}
}
