/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.replacestring;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * Listen to changes in list of files
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
class ListSelectionActionListener implements ListSelectionListener,
		SelectedItensRetreiver {

	// List of files that are selected
	private ArrayList selectedElements = new ArrayList();

	// Optional listener to any changes on the selection of
	// the file list.
	private ItensSelectionListener itensSelectionListener;

	private DefaultListModel listModel;

	private Vector someItensSelectionListeners = new Vector();
	private Vector noItensSelectionListeners = new Vector();

	/**
	 * Constructor. It is necessary to inform a model.
	 */
	ListSelectionActionListener(DefaultListModel newListModel) {
		listModel = newListModel;
	}

	/**
	 * Optional add of a listener of changes on any selection from the list
	 */
	public void addSomeItensSelectionListener(
			ItensSelectionListener newitensSelectionListener) {
		someItensSelectionListeners.add(newitensSelectionListener);
	}

	/**
	 * Optional remove of a listener of changes on any selection from the list
	 */
	public void removeSomeItensSelectionListener(
			ItensSelectionListener newitensSelectionListener) {
		someItensSelectionListeners.remove(newitensSelectionListener);
	}

	private void triggerSomeItensSelectionEvent() {
		Enumeration listenersEnumeration = someItensSelectionListeners
				.elements();
		while (listenersEnumeration.hasMoreElements()) {
			ItensSelectionListener listener = (ItensSelectionListener) listenersEnumeration
					.nextElement();
			listener.someItensSelected();
		}
	}

	/**
	 * Optional add of a listener of changes on any selection from the list
	 */
	public void addNoItensSelectionListener(
			ItensSelectionListener newitensSelectionListener) {
		noItensSelectionListeners.add(newitensSelectionListener);
	}

	/**
	 * Optional remove of a listener of changes on any selection from the list
	 */
	public void removeNoItensSelectionListener(
			ItensSelectionListener newitensSelectionListener) {
		noItensSelectionListeners.remove(newitensSelectionListener);
	}

	private void triggerNoItensSelectionEvent() {
		Enumeration listenersEnumeration = noItensSelectionListeners.elements();
		while (listenersEnumeration.hasMoreElements()) {
			ItensSelectionListener listener = (ItensSelectionListener) listenersEnumeration
					.nextElement();
			listener.noItensSelected();
		}
	}

	/**
	 * Return the selected files on the list
	 */
	public ArrayList getSelectedFiles() {

		ArrayList fileNames = new ArrayList();

		Iterator selectedItens = selectedElements.iterator();

		while (selectedItens.hasNext()) {
			Integer index = (Integer) selectedItens.next();

			fileNames.add(listModel.get(index.intValue()));
		}

		return fileNames;
	}

	public void valueChanged(ListSelectionEvent e) {

		ListSelectionModel listSelectionModel = null;

		listSelectionModel = (ListSelectionModel) e.getSource();

		if (listSelectionModel.isSelectionEmpty()) {
			selectedElements = new ArrayList();
			triggerNoItensSelectionEvent();
		} else {
			// Collect the selected itens
			boolean someItensSelected = false;

			selectedElements = new ArrayList();
			int minIndex = listSelectionModel.getMinSelectionIndex();
			int maxIndex = listSelectionModel.getMaxSelectionIndex();
			for (int i = minIndex; i <= maxIndex; i++) {
				if (listSelectionModel.isSelectedIndex(i)) {
					someItensSelected = true;
					selectedElements.add(new Integer(i));
				}
			}

			triggerSomeItensSelectionEvent();

		}
	}
}
