/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

/**
 * Link all the components and start the gui.
 */

package com.carlosbcruz.replacestring;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.DefaultListModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;

/**
 * Main form.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class MainForm extends JFrame implements FinishedReadingListener {

	static MainForm mainForm;

	// The desktop classes
	JDesktopPane desktopPane;

	ListSelectionActionListener listSelectionActionListener;

	SelectActionResponse selectAction;

	DefaultListModel listModel;

	/**
	 * Constructor, build the gui.
	 */
	MainForm() {

		super("String Replacement");

		buildMainForm();

		// Listem to the windows closing event.
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});

		pack();
		setVisible(true);
	}

	public void finishedReadingAction() {
		listModel.removeAllElements();
		ArrayList list = selectAction.getInternalDocuments();
		for (int i = 0; i < list.size(); i++) {
			listModel.addElement(((InternalDocument) list.get(i)).getTitle());
		}
	}

	/**
	 * Build the gui
	 */
	private void buildMainForm() {

		// Instantiate and link menu with listeners
		MainMenu mainMenu = new MainMenu();
		mainMenu.addExitActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});

		// Create a default list selection model and attach to the menu
		// class to listen to the file changes to enable or not an
		// option on the menu.

		// Create the first Model and attach the listener to any
		// change on file selections
		listModel = new DefaultListModel();
		listSelectionActionListener = new ListSelectionActionListener(listModel);
		listSelectionActionListener.addSomeItensSelectionListener(mainMenu);
		listSelectionActionListener.addNoItensSelectionListener(mainMenu);

		// Open the file dialog and update the listModel with the
		// files that are not currently selected.
		selectAction = new SelectActionResponse(this);

		// Make the element appear at the list when it is readed
		selectAction.addFinishedReadingListeners(this);

		selectAction.addHasFilesListeners(mainMenu);
		mainMenu.addSelectActionListener(selectAction);

		// Create the second Model and attach the same listener above
		// to listen to the changes of selection.
		DefaultListSelectionModel listSelectionModel = new DefaultListSelectionModel();
		listSelectionModel
				.addListSelectionListener(listSelectionActionListener);

		// Attach the two models one for the files and another for
		// the selected files.
		JList jlist = new JList(listModel);
		jlist.setSelectionModel(listSelectionModel);

		/**
		 * Get the information to replace string and generate copies of the
		 * files with the requested strings replaced.
		 */
		mainMenu.addReplaceStringActionListener(new ActionListener() {

			ReplaceActionResponse response = null;

			public void actionPerformed(ActionEvent evt) {

				// Create a modal dialog to enter the string to be replaced
				final JDialog dialog = new JDialog(mainForm,
						"String Replace Information", true);

				Container content = dialog.getContentPane();

				// Set the layout
				content.setLayout(new GridLayout(0, 2));

				// Insert the components
				JLabel fromLabel = new JLabel("Replace:");
				JLabel byLabel = new JLabel("By:");
				final JTextField fromText = new JTextField();
				final JTextField byText = new JTextField();
				JButton okButton = new JButton("Replace on All Files");

				// Listem to ok Button
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						response = new ReplaceActionResponse(mainForm,
								selectAction.getInternalDocuments(), fromText
										.getText(), byText.getText());
						response.actionPerformed(event);
						dialog.setVisible(false);
					}
				});

				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						dialog.setVisible(false);
					}
				});

				content.add(fromLabel);
				content.add(fromText);
				content.add(byLabel);
				content.add(byText);
				content.add(okButton);
				content.add(cancelButton);

				dialog.pack();

				Point mainLocation = mainForm.getLocation();
				Point newLocation = new Point(
						(int) (mainLocation.getX() + (mainForm.getWidth() - dialog
								.getWidth()) / 2),
						(int) (mainLocation.getY() + (mainForm.getHeight() - dialog
								.getHeight()) / 2));
				dialog.setLocation(newLocation);
				dialog.setVisible(true);

			}
		});

		/**
		 * Save all the files
		 */
		mainMenu.addSaveActionListener(new ActionListener() {

			SaveActionResponse response = null;

			public void actionPerformed(ActionEvent evt) {

				// Create a modal dialog to enter the target directory
				final JDialog dialog = new JDialog(mainForm,
						"Save Files Information", true);

				Container content = dialog.getContentPane();

				// Set the layout
				content.setLayout(new GridLayout(0, 3));

				// Insert the components
				JLabel targetDirectoryLabel = new JLabel("Target Directory:");
				final JTextField targetDirectory = new JTextField();
				JButton browseDirectory = new JButton("Browse Directories");
				JButton okButton = new JButton("Save All Files");

				// Listem to ok Button
				browseDirectory.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {

						JFileChooser jFileChooser = new JFileChooser();
						jFileChooser
								.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
						jFileChooser.setMultiSelectionEnabled(false);

						int returnValue = jFileChooser.showOpenDialog(dialog);

						if (returnValue == JFileChooser.APPROVE_OPTION) {

							targetDirectory.setText(jFileChooser
									.getSelectedFile().toString());
						}
					}
				});

				// Listem to ok Button
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						File directory = new File(targetDirectory.getText());
						if (!directory.exists()) {
							JOptionPane.showMessageDialog(mainForm,
									"The specified directory does not exists",
									"Warning", JOptionPane.WARNING_MESSAGE);
							dialog.setVisible(false);
							return;
						}

						response = new SaveActionResponse(mainForm,
								selectAction.getInternalDocuments(), directory
										.getAbsolutePath());
						response.actionPerformed(event);
						dialog.setVisible(false);

					}
				});

				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						dialog.setVisible(false);
					}
				});

				content.add(targetDirectoryLabel);
				content.add(targetDirectory);
				content.add(browseDirectory);
				content.add(new JLabel());
				content.add(okButton);
				content.add(cancelButton);

				dialog.pack();

				Point mainLocation = mainForm.getLocation();
				Point newLocation = new Point(
						(int) (mainLocation.getX() + (mainForm.getWidth() - dialog
								.getWidth()) / 2),
						(int) (mainLocation.getY() + (mainForm.getHeight() - dialog
								.getHeight()) / 2));
				dialog.setLocation(newLocation);
				dialog.setVisible(true);

			}
		});

		/**
		 * Add the show files class to work when the Show Files menu item is
		 * selected.
		 */
		mainMenu.addShowFilesActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {

				// Create all the internal frames with the documents.
				ArrayList internalDocuments = selectAction
						.getInternalDocuments();

				ArrayList internalSelectedDocuments = listSelectionActionListener
						.getSelectedFiles();

				Iterator internalDocumentsSelectedIterator = internalSelectedDocuments
						.iterator();
				while (internalDocumentsSelectedIterator.hasNext()) {

					String documentName = (String) internalDocumentsSelectedIterator
							.next();

					Iterator internalDocumentsIterator = internalDocuments
							.iterator();

					while (internalDocumentsIterator.hasNext()) {
						InternalDocument document = (InternalDocument) internalDocumentsIterator
								.next();

						if (document.getTitle().equals(documentName)) {

							TextInternalFrame internalFrame = new TextInternalFrame(
									document.getTitle(), document.getContent());
							document.setInternalFrame(internalFrame);
							desktopPane.add(internalFrame);
							internalFrame.pack();
							internalFrame.setVisible(true);
						}
					}
				}
			}
		});

		mainMenu.addAboutActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				JOptionPane.showMessageDialog(MainForm.this,
						"This App was Creted by Carlos F Bella Cruz.");
			}
		});

		// Scroll the JList.
		JScrollPane listScroolPane = new JScrollPane(jlist);

		// Create the desktop to receive internal windows
		desktopPane = new JDesktopPane();
		desktopPane.setPreferredSize(new Dimension(300, 300));

		// Make the split window and add to a Panel
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				listScroolPane, new JScrollPane(desktopPane));

		// Glue the SplitPane with the JPanel.
		JPanel globalPanel = new JPanel();
		globalPanel.setLayout(new BorderLayout());
		globalPanel.add(splitPane, BorderLayout.CENTER);
		setContentPane(globalPanel);

		setJMenuBar(mainMenu);

		// Create a toolbar
		JToolBar toolBar = new JToolBar();

		MainToolBar mainToolBar = new MainToolBar();
		mainToolBar.addFileSelectActionListener(selectAction);
		selectAction.addHasFilesListeners(mainToolBar);

		// Add Toolbar to the panel
		globalPanel.add(mainToolBar, BorderLayout.NORTH);

	}

	public static void main(String[] args) {
		mainForm = new MainForm();
	}
}
