/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

/**
 * Listem to the "Select Files" menu item action
 */

package com.carlosbcruz.replacestring;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JOptionPane;
import javax.swing.ProgressMonitor;

/**
 * Replace action response.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class ReplaceActionResponse implements ActionListener {

	ArrayList internalDocuments;

	final private Component superiorComponent;

	// Strings to find and to replace
	private String from, by;

	// Step to show the time progress
	public final static int HALF_SECOND = 500;
	// Progress bar
	javax.swing.Timer timer;
	ProgressMonitor progressMonitor;

	StringReplacer stringReplacer;

	/**
	 * Constructor that receives the superior component to allow the usage of
	 * File Open Window and the DefaultListModel to insert the files selected by
	 * the user.
	 */
	ReplaceActionResponse(Component newSuperiorComponent,
			ArrayList internalDocuments, String from, String by) {

		superiorComponent = newSuperiorComponent;
		this.internalDocuments = internalDocuments;
		this.from = from;
		this.by = by;
	}

	/**
	 * Print the progress message of files reading
	 */
	class TimerListener implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			if (progressMonitor.isCanceled() || stringReplacer.done()) {
				progressMonitor.close();
				stringReplacer.stop();
				timer.stop();

				long numOfReplaces = stringReplacer.getNumberOfChanges();

				JOptionPane.showMessageDialog(null, "Number of Replaces = "
						+ numOfReplaces, "Information",
						JOptionPane.INFORMATION_MESSAGE);

				// Add the documents readed into the memory.
				// internalDocuments = stringReplacer.getNewIntarnalDocuments();

			} else {
				progressMonitor.setNote(stringReplacer.getMessage());
				progressMonitor.setProgress((int) ((double) stringReplacer
						.getCurrent()
						/ (double) stringReplacer.getLengthOfTask() * 100.0));
			}
		}
	}

	/**
	 * Insert the files selected in the DefaultListModel
	 */
	public void actionPerformed(ActionEvent e) {

		if (from == null)
			from = new String();
		if (by == null)
			by = new String();

		if (superiorComponent == null || internalDocuments.isEmpty()
				|| from.equals("")) {
			if (from.equals(""))
				JOptionPane.showMessageDialog(null,
						"No String informed for replacement", "Warning",
						JOptionPane.WARNING_MESSAGE);

			return;
		}

		// Create a document reader to the file names.
		stringReplacer = new StringReplacer(internalDocuments, from, by);

		// Load the files.
		progressMonitor = new ProgressMonitor(superiorComponent, "Loading ",
				"", 0, 100);
		progressMonitor.setProgress(0);
		stringReplacer.go();

		timer = new javax.swing.Timer(HALF_SECOND, new TimerListener());

		timer.start();

	}

	/**
	 * Returns the internalDocuments.
	 * 
	 * @return ArrayList
	 */
	public ArrayList getInternalDocuments() {
		return internalDocuments;
	}

}
