/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

/**
 * Listem to the "Select Files" menu item action
 */

package com.carlosbcruz.replacestring;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.ProgressMonitor;

/**
 * Selected action response.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class SelectActionResponse implements ActionListener,
		DocumentCreationListener {

	ArrayList internalDocuments = new ArrayList();
	ArrayList duplicatedInternalDocuments = new ArrayList();

	final private Component superiorComponent;

	// Step to show the time progress
	public final static int HALF_SECOND = 500;
	// Progress bar
	javax.swing.Timer timer;
	ProgressMonitor progressMonitor;

	DocumentsReader documentsReader;

	Vector hasFilesListeners = new Vector();

	Vector finishedReadingListeners = new Vector();

	/**
	 * Constructor that receives the superior component to allow the usage of
	 * File Open Window and the DefaultListModel to insert the files selected by
	 * the user.
	 */
	SelectActionResponse(Component newSuperiorComponent) {

		superiorComponent = newSuperiorComponent;
	}

	public void addFinishedReadingListeners(FinishedReadingListener listener) {
		finishedReadingListeners.add(listener);
	}

	public void removeFinishedReadingListeners(FinishedReadingListener listener) {
		finishedReadingListeners.remove(listener);
	}

	private void triggerFinishedReading() {
		Enumeration listenerEnumeration = finishedReadingListeners.elements();
		while (listenerEnumeration.hasMoreElements()) {
			FinishedReadingListener listener = (FinishedReadingListener) listenerEnumeration
					.nextElement();
			listener.finishedReadingAction();
		}
	}

	public void addHasFilesListeners(HasFilesListener hasFilesListener) {
		hasFilesListeners.add(hasFilesListener);
	}

	public void removeHasFilesListeners(HasFilesListener hasFilesListener) {
		hasFilesListeners.remove(hasFilesListener);
	}

	private void triggerHasFilesEvent() {
		Enumeration listenerEnumeration = hasFilesListeners.elements();
		while (listenerEnumeration.hasMoreElements()) {
			HasFilesListener listener = (HasFilesListener) listenerEnumeration
					.nextElement();
			listener.hasFiles();
		}
	}

	public void documentReaded(InternalDocument document) {

		// Check if it is duplicated
		for (int i = 0; i < internalDocuments.size(); i++)
			if (((InternalDocument) internalDocuments.get(i)).getTitle()
					.equals(document.getTitle())) {
				duplicatedInternalDocuments.add(document);
				return;
			}

		internalDocuments.add(document);
	}

	/**
	 * Print the progress message of files reading
	 */
	class TimerListener implements ActionListener {

		public void actionPerformed(ActionEvent evt) {

			if (progressMonitor.isCanceled() || documentsReader.done()) {
				timer.stop();
				documentsReader.stop();
				progressMonitor.close();

				String names = new String();
				for (int i = 0; i < duplicatedInternalDocuments.size(); i++) {
					names += ((InternalDocument) duplicatedInternalDocuments
							.get(i)).getTitle()
							+ "\n";
				}

				if (duplicatedInternalDocuments.size() > 0) {
					JOptionPane
							.showMessageDialog(superiorComponent,
									"The Following files are already readed:\n"
											+ names, "Warning",
									JOptionPane.WARNING_MESSAGE);
				}

				triggerFinishedReading();

			} else {
				progressMonitor.setNote(documentsReader.getMessage());
				progressMonitor.setProgress((int) ((double) documentsReader
						.getCurrent()
						/ (double) documentsReader.getLengthOfTask() * 100.0));
			}
		}
	}

	/**
	 * Check if there is documents to read and perform the reading if necessary
	 */
	public void readDocuments(ArrayList fileNamesToRead) {

		if (fileNamesToRead.isEmpty())
			return;

		triggerHasFilesEvent();

		// Create a document reader to the file names.
		documentsReader = new DocumentsReader(fileNamesToRead);

		documentsReader.addDocumentoCreationListener(this);

		// Load the files.
		progressMonitor = new ProgressMonitor(superiorComponent, "Loading ",
				"", 0, 100);
		progressMonitor.setProgress(0);
		documentsReader.go();

		timer = new javax.swing.Timer(HALF_SECOND, new TimerListener());

		timer.start();
	}

	/**
	 * Insert the files selected in the DefaultListModel
	 */
	public void actionPerformed(ActionEvent e) {

		if (superiorComponent == null)
			return;

		JFileChooser jFileChooser = new JFileChooser();
		jFileChooser.setMultiSelectionEnabled(true);

		ArrayList fileNamesToRead = new ArrayList();

		int returnValue = jFileChooser.showOpenDialog(superiorComponent);

		if (returnValue == JFileChooser.APPROVE_OPTION) {

			File[] files = jFileChooser.getSelectedFiles();

			ArrayList alreadyInList = new ArrayList();
			for (int i = 0; i < files.length; i++) {
				fileNamesToRead.add(files[i].toString());
			}
		}

		readDocuments(fileNamesToRead);

	}

	/**
	 * Returns the internalDocuments.
	 * 
	 * @return ArrayList
	 */
	public ArrayList getInternalDocuments() {
		return internalDocuments;
	}

}