/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.replacestring;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Replace the strings on all texts
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class StringReplacer {

	// The number of bytes to read
	private long lengthOfTask;

	// Actual file that is being read
	private long current;

	// Number of strings replaced
	private long numberOfChanges = 0;

	// Message of the current status.
	private String statMessage;

	// Internal Windows that are generated to the files.
	private ArrayList intarnalDocuments;
	private ArrayList newIntarnalDocuments = new ArrayList();

	// List of files to work
	private ArrayList internalDocuments;

	boolean stop = false;

	private String from, by;

	/**
	 * Constructor. Get the file list and count the total number of bytes.
	 */
	StringReplacer(ArrayList internalDocuments, String from, String by) {

		// Get the file names
		this.internalDocuments = internalDocuments;
		this.from = from;
		this.by = by;

		// Count the number of bytes to read
		long counter = 0;
		Iterator documentsIterator = internalDocuments.iterator();
		while (documentsIterator.hasNext()) {
			InternalDocument documentfile = (InternalDocument) documentsIterator
					.next();
			counter += documentfile.getContent().length();
		}
		lengthOfTask = counter;
	}

	/**
	 * Start the process of reading the files through an inner class ActualTask.
	 */
	void go() {
		current = 0;
		final SwingWorker worker = new SwingWorker() {
			public Object construct() {
				return new ActualTask();
			}
		};

		worker.start();
	}

	/**
	 * Get the total number of bytes to read
	 */
	long getLengthOfTask() {
		return lengthOfTask;
	}

	/**
	 * Get the total number of bytes readed until the moment of this method
	 * call.
	 */
	long getCurrent() {
		return current;
	}

	/**
	 * Stop the reading
	 */
	void stop() {
		stop = true;
		current = lengthOfTask;
	}

	/**
	 * Inform if the read is completed
	 */
	boolean done() {

		return (current >= lengthOfTask);
	}

	/**
	 * Return the current message status
	 */
	String getMessage() {
		return statMessage;
	}

	/**
	 * Class to perform the reading
	 */
	class ActualTask {

		/**
		 * Constructor to perform the reading
		 */
		ActualTask() {

			current = 0;
			stop = false;
			numberOfChanges = 0;

			for (int f = 0; f < internalDocuments.size(); f++) {
				InternalDocument document = (InternalDocument) internalDocuments
						.get(f);

				StringBuffer content = document.getContent();
				StringBuffer newContent = new StringBuffer();

				int i = 0;
				do {

					boolean found = true;
					for (int j = 0; j < from.length(); j++) {
						if (content.charAt(i + j) != from.charAt(j)) {
							found = false;
							break;
						}
					}

					if (stop)
						return;

					if (found) {
						newContent.append(by);
						i += from.length();
						current += from.length();
						numberOfChanges++;
					} else {
						newContent.append(content.charAt(i));
						i++;
						current += 1;
					}

				} while (i < content.length() - by.length());
				statMessage = "Completed " + current + " of " + lengthOfTask;
				do {
					newContent.append(content.charAt(i));
					i++;
				} while (i < content.length());
				document.setContent(newContent);
				current += by.length();
			}
		}

		/**
		 * Returns the newIntarnalDocuments.
		 * 
		 * @return ArrayList
		 */
		public ArrayList getNewIntarnalDocuments() {
			return newIntarnalDocuments;
		}

	}

	/**
	 * Returns the numberOfChanges.
	 * 
	 * @return long
	 */
	public long getNumberOfChanges() {
		return numberOfChanges;
	}

}
