/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.resourceusage;

/**
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

import java.util.ArrayList;

import com.carlosbcruz.util.ArraySupport;
import com.carlosbcruz.util.FileSupport;
import com.carlosbcruz.util.Out;
import com.carlosbcruz.util.SimpleProperties;
import com.carlosbcruz.util.StringSupport;
import com.carlosbcruz.util.Task;
import com.carlosbcruz.util.TaskController;
import com.carlosbcruz.util.TimeSupport;

/**
 * Verify if how each resource token has been used in the project
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class VerifyResourceUsage {

	/**
	 * Represents the task to generate the unused content keys
	 * 
	 * @author Carlos Fernando Bella Cruz - <a
	 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
	 */
	class MyTask extends Task {

		public MyTask(String taskName, Object parameter) {

			super(taskName);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see com.carlosbcruz.util.Task#run(java.lang.Object)
		 */
		public Object run(Object parameter) throws InterruptedException {

			setSteps(3);

			TimeSupport.startChronometer();

			String contentLocation = SimpleProperties
					.confirmFileNamePropertyWithDefault(
							"content.file.location",
							"C:/RAD7_TaxInquire/taxinq/properties/SQLBCIT.properties",
							"Content file name.");
			Out.println("Content location: " + contentLocation);

			verifyInterrupted();

			StringBuffer rawfileContent = FileSupport
					.readTextFile(contentLocation);

			StringBuffer keysOnlyFileContent = contentToKeyList(rawfileContent);

			verifyInterrupted();

			String keysFileLocation = SimpleProperties
					.confirmFileNamePropertyWithDefault(
							"keys.temp.file.location", "c:\\content.txt",
							"Temporary file name.");
			Out.println("Keys file location: " + keysFileLocation);
			// If there is an error message than return null.
			if (FileSupport.writeTextFile(keysFileLocation, keysOnlyFileContent
					.toString()) != null)
				return null;

			verifyInterrupted();

			String lines[] = FileSupport.readAllLines(keysFileLocation);

			String workspaceLocation = SimpleProperties
					.confirmDirectoryNamePropertyWithDefault(
							"workspace.root.location", "C:/RAD7_TaxInquire",
							"Workspace root file.");
			Out.println("Workspace location: " + workspaceLocation);

			verifyInterrupted();

			String files[] = FileSupport.getFileOnSubfolder(workspaceLocation,
					null);

			String javaFiles[] = ArraySupport.filterBySufix(files, "java");

			String jspFiles[] = ArraySupport.filterBySufix(files, "jsp");

			Object targetFiles[] = ArraySupport
					.mergeArrays(javaFiles, jspFiles);

			setUpperLimit(lines.length);

			verifyInterrupted();

			// For each token
			for (int i = 0; i < lines.length; i++) {

				setCurrentProgress(i);
				verifyInterrupted();

				Out.println(lines[i]);

				boolean tokenIsUsed = false;
				// For each file
				for (int j = 0; j < targetFiles.length; j++) {

					String fileName = (String) targetFiles[j];

					StringBuffer fileContent = FileSupport
							.readTextFile(fileName);

					int ocurrences[] = StringSupport.findOcurrences(
							fileContent, lines[i]);

					if (ocurrences != null && ocurrences.length > 0) {

						tokenIsUsed = true;

						Out.println("          " + ocurrences.length + "  "
								+ fileName);
					}
				}

				if (!tokenIsUsed) {
					Out.println("Token not used " + lines[i] + ".");
				}
			}

			String lines1[] = FileSupport.readAllLines(Out.getFileName());

			String lines2[] = FileSupport.readAllLines(contentLocation);

			setUpperLimit(lines1.length);

			setCurrentStep(2);

			ArrayList unusedTokens = new ArrayList();

			int counter = 0;
			for (int i = 0; i < lines1.length; i++) {

				setCurrentProgress(i);
				verifyInterrupted();

				int start = lines1[i].indexOf("Token not used ");
				int end = lines1[i].indexOf(".");

				if (start != -1) {
					String token = lines1[i].substring(start
							+ "Token not used ".length(), end);
					unusedTokens.add(token);

					for (int j = 0; j < lines2.length; j++) {
						int location = lines2[j].indexOf(token);
						if (location != -1) {
							Out.println((++counter) + " - " + lines2[j]);
						}
					}
				}

			}

			String contentLines[] = FileSupport.readAllLines(contentLocation);

			Out.println("Tokens used");

			setCurrentStep(3);
			setUpperLimit(contentLines.length);

			for (int i = 0; i < contentLines.length; i++) {

				setCurrentProgress(i);

				boolean isUnused = false;
				for (int j = 0; j < unusedTokens.size(); j++) {
					String token = (String) unusedTokens.get(j);

					int k = contentLines[i].indexOf(token);

					verifyInterrupted();

					if (k != -1)
						isUnused = true;
				}

				if (!isUnused) {
					Out.println(contentLines[i]);
				}
			}

			TimeSupport.stopChronometer();

			Out.println(TimeSupport.getExecutionTime());

			return "All Done";
		}
	}

	/**
	 * Convert a content file to a list of keys only
	 * 
	 * @param content
	 *            the content file
	 * @return the list of keys
	 */
	public static StringBuffer contentToKeyList(StringBuffer content) {

		StringBuffer output = new StringBuffer();
		boolean replace = true;
		for (int i = 0; i < content.length(); i++) {

			char nextChar = content.charAt(i);

			if (nextChar == '\n')
				replace = true;

			if (replace == true) {
				if (nextChar == '=' || nextChar == '#') {
					replace = false;
					continue;
				} else {
					output.append(nextChar);
				}
			}
		}

		content = output;

		ArrayList lines = new ArrayList();
		StringBuffer line = new StringBuffer();
		for (int i = 0; i < content.length(); i++) {

			char nextChar = content.charAt(i);

			if (nextChar == '\n') {
				String singleLine = line.toString().trim();
				if (!singleLine.equals("")) {
					lines.add(singleLine);
				}
				line = new StringBuffer();
			} else {
				line.append(nextChar);
			}
		}

		output = new StringBuffer();

		for (int i = 0; i < lines.size(); i++) {
			output.append(lines.get(i));
			output.append("\n");

		}

		return output;
	}

	/**
	 * Execute the Resource usage classes
	 */
	private void runTask() {

		TaskController controller = new TaskController("Tasks");
		controller.addFiniteTask(new MyTask("Unused content keys", new Boolean(
				true)));
		controller.start();

		controller.getResults(500);
	}

	/**
	 * Main class.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		VerifyResourceUsage verifyResourceUsage = new VerifyResourceUsage();
		verifyResourceUsage.runTask();
	}
}
