/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.study;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

import com.carlosbcruz.util.ExceptionSupport;
import com.carlosbcruz.util.FileSupport;
import com.carlosbcruz.util.Serialization;

/**
 * The main Study frame.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class Study extends JFrame {

	private static final long serialVersionUID = 1L;

	private ArrayList<Word> words = new ArrayList<Word>();
	private String answer = new String();
	private int cursor = -1;
	private int studyCursor = -1;
	private int correctAnswer = -1;
	private int totalScore = 0;
	private int currentWordScore = 10;
	private int record = -1;

	private static final String STUDY_FILE_NAME = "Study.ser";

	private JTabbedPane tabPane = new JTabbedPane();

	private JLabel wordNumber = new JLabel(String.valueOf(cursor));

	private JTextField englishField = new JTextField(50);
	private JTextField portugueseField = new JTextField(50);

	private JButton insert = new JButton("Inserir");
	private JButton delete = new JButton("Remover");
	private JButton next = new JButton("Prxima");
	private JButton previous = new JButton("Anterior");

	private JLabel portugueseWord = new JLabel();
	private JLabel result = new JLabel();
	private JLabel correctTypedWord = new JLabel();
	private JTextField typedWord = new JTextField(1);
	private JLabel congratulations = new JLabel();

	private JLabel totalScoreLabel = new JLabel();
	private JLabel currentWordScoreLabel = new JLabel();

	private JLabel greatestScore = new JLabel();

	Study() {

		portugueseWord.setFont(new Font("Arial", 1, 42));
		portugueseWord.setForeground(new Color(0, 0, 255));

		result.setFont(new Font("Arial", 1, 42));

		correctTypedWord.setFont(new Font("Arial", 1, 42));
		correctTypedWord.setForeground(new Color(255, 0, 0));

		congratulations.setFont(new Font("Arial", 1, 42));
		congratulations.setForeground(new Color(0, 255, 0));

		totalScoreLabel.setFont(new Font("Arial", 1, 42));
		totalScoreLabel.setForeground(new Color(255, 255, 0));

		currentWordScoreLabel.setFont(new Font("Arial", 1, 24));

		currentWordScoreLabel.setText("Valendo: " + currentWordScore);

		greatestScore.setText("Recorde: " + 0);

		greatestScore.setFont(new Font("Arial", 1, 42));
		greatestScore.setForeground(new Color(0, 255, 255));

		loadWords();

		JPanel studyPanel = new JPanel(new GridLayout(8, 1));
		if (studyCursor >= 0) {

			Word newWord = words.get(studyCursor);

			portugueseWord.setText(newWord.getPortugueseWord());
			answer = newWord.getEnglishWord();
		}

		studyPanel.add(portugueseWord);
		studyPanel.add(result);

		studyPanel.add(correctTypedWord);
		studyPanel.add(typedWord);
		studyPanel.add(congratulations);

		studyPanel.add(totalScoreLabel);
		studyPanel.add(currentWordScoreLabel);
		studyPanel.add(greatestScore);

		typedWord.addKeyListener(new KeyListener() {

			@Override
			public void keyPressed(KeyEvent e) {

				System.out.println(e.getKeyChar());

				Character nextChar = answer.charAt(correctAnswer + 1);

				if (e.getKeyChar() == nextChar) {
					result.setText("Acertou");
					correctAnswer++;
				} else {
					result.setText("Errou");

					currentWordScore--;
					if (currentWordScore < 0) {
						currentWordScore = 0;
					}
					currentWordScoreLabel.setText("Valendo: "
							+ currentWordScore);

				}

				if (correctAnswer + 1 >= answer.length()) {
					studyCursor++;

					congratulations.setText("Parabns! "
							+ portugueseWord.getText() + "  " + answer);

					totalScore += currentWordScore;

					if (studyCursor == words.size()) {

						if (totalScore > record) {
							greatestScore.setText("Recorde: " + totalScore);
						}
						totalScore = 0;
						totalScoreLabel.setText("Pontos: " + totalScore);
						studyCursor = 0;
					}

					Word word = words.get(studyCursor);
					portugueseWord.setText(word.getPortugueseWord());
					answer = word.getEnglishWord();
					correctTypedWord.setText("");
					correctAnswer = -1;

					totalScoreLabel.setText("Pontos: " + totalScore);
					currentWordScore = 10;
					currentWordScoreLabel.setText("Valendo: "
							+ currentWordScore);

				} else {

					if (correctAnswer >= 0) {
						correctTypedWord.setText(answer.substring(0,
								correctAnswer + 1));
					}
				}

				typedWord.setText("");

			}

			@Override
			public void keyReleased(KeyEvent e) {

			}

			@Override
			public void keyTyped(KeyEvent e) {

			}
		});

		JPanel wordsPanel = new JPanel();
		wordsPanel.setLayout(new BorderLayout());

		JPanel internalPanel = new JPanel();
		// wordsPanel.setLayout(new FlowLayout());
		internalPanel.setLayout(new GridLayout(5, 2));

		internalPanel.add(new JLabel("Numero:"));
		internalPanel.add(wordNumber);

		internalPanel.add(new JLabel("Ingls:"));
		internalPanel.add(englishField);
		internalPanel.add(new JLabel("Portugus:"));
		internalPanel.add(portugueseField);

		internalPanel.add(insert);
		internalPanel.add(delete);
		internalPanel.add(previous);
		internalPanel.add(next);

		insert.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				Word newWord = new Word();
				newWord.setEnglishWord(englishField.getText());
				newWord.setPortugueseWord(portugueseField.getText());

				words.add(newWord);

				cursor++;

				wordNumber.setText(String.valueOf(cursor));
			}

		});

		delete.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				if (cursor >= 0) {

					words.remove(cursor);
					cursor--;

					if (cursor >= 0) {

						Word newWord = words.get(cursor);
						englishField.setText(newWord.getEnglishWord());
						portugueseField.setText(newWord.getPortugueseWord());
					} else {
						englishField.setText("");
						portugueseField.setText("");

					}
				}

				wordNumber.setText(String.valueOf(cursor));
			}

		});

		next.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				if (cursor >= words.size() - 1) {
					cursor = -1;
				}

				if (cursor < words.size()) {
					cursor++;
					Word newWord = words.get(cursor);
					englishField.setText(newWord.getEnglishWord());
					portugueseField.setText(newWord.getPortugueseWord());
				}

				wordNumber.setText(String.valueOf(cursor));
			}

		});

		previous.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				if (cursor <= 0) {
					cursor = words.size();
				}

				if (cursor >= 0) {

					cursor--;
					Word newWord = words.get(cursor);
					englishField.setText(newWord.getEnglishWord());
					portugueseField.setText(newWord.getPortugueseWord());
				}

				wordNumber.setText(String.valueOf(cursor));
			}

		});

		wordsPanel.add(internalPanel, BorderLayout.NORTH);

		tabPane.addTab("Estudo", studyPanel);
		tabPane.addTab("Cadastro", wordsPanel);

		setContentPane(tabPane);

		addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent arg0) {

				saveWords();
				System.exit(ABORT);
			}
		});

		Dimension screenDimension = Toolkit.getDefaultToolkit().getScreenSize();
		// Set window size and position
		setBounds(0, 0, (int) screenDimension.getWidth(), (int) screenDimension
				.getHeight());

		setVisible(true);

	}

	private void saveWords() {

		try {
			Serialization.serializeObject(STUDY_FILE_NAME, words);
			Serialization.serializeObject(FileSupport
					.getCurrentDateForFileName()
					+ STUDY_FILE_NAME, words);
		} catch (Exception e) {
			ExceptionSupport.handleException(e);
		}
	}

	private void loadWords() {

		try {
			File file = new File(STUDY_FILE_NAME);

			if (file.exists()) {
				words = (ArrayList<Word>) Serialization
						.deserializeObject(STUDY_FILE_NAME);

				if (words.size() > 0) {
					studyCursor = 0;
				}

			}
		} catch (Exception e) {
			ExceptionSupport.handleException(e);
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		Study study = new Study();
	}

}
