/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.timesheet;

import info.monitorenter.gui.chart.Chart2D;
import info.monitorenter.gui.chart.ITrace2D;
import info.monitorenter.gui.chart.traces.Trace2DSimple;

import java.awt.Color;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.StringTokenizer;

import javax.swing.JFrame;

import com.carlosbcruz.util.FileSupport;
import com.carlosbcruz.util.StringSupport;

/**
 * Analyze a timesheet file.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class Timesheet {

	/**
	 * Generate a chart of use
	 */
	public static void generateChart(ArrayList<PeriodBean> periods) {

		Chart2D chart = new Chart2D();

		ITrace2D trace = new Trace2DSimple("Periods");

		chart.addTrace(trace);
		chart.setForeground(new Color(0, 0, 255));

		int i = 0;
		trace.addPoint(i++, 0);
		for (PeriodBean period : periods) {
			trace.addPoint(i++, period.getDuration());
		}

		JFrame frame = new JFrame("Timesheet");
		frame.getContentPane().add(chart);
		frame.setSize(400, 300);
		frame.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {

				System.exit(0);
			}
		});
		frame.setVisible(true);
	}

	/**
	 * Convert a date from format DOS US "Thu 04/01/2010" to Brazil
	 * 
	 * @param date
	 *            a date with a format "Thu 04/01/2010"
	 * @return a date with brazilian format.
	 */
	private static String convertDate(String date) {

		String conversionTable[][] = { { "Sun", "Domingo" },
				{ "Mon", "Segunda" }, { "Tue", "Tera" }, { "Wed", "Quarta" },
				{ "Thu", "Quinta" }, { "Fri", "Sexta" }, { "Sat", "Sbado" }, };

		StringTokenizer dateTokenizer = new StringTokenizer(date);

		String dayOfTheWeek = dateTokenizer.nextToken().trim();

		String dayName = new String("");
		for (int i = 0; i < conversionTable.length; i++) {
			if (conversionTable[i][0].equals(dayOfTheWeek)) {
				dayName = conversionTable[i][1];
				break;
			}
		}

		StringTokenizer datePartsTokenazer = new StringTokenizer(dateTokenizer
				.nextToken().trim(), "/");

		String month = datePartsTokenazer.nextToken().trim();
		String day = datePartsTokenazer.nextToken().trim();
		String year = datePartsTokenazer.nextToken().trim();

		return day + "/" + month + "/" + year
				+ StringSupport.setMinimumLengthRight(" (" + dayName + ")", 10);
	}

	/**
	 * Process a timesheet file.
	 * 
	 * @param periods
	 *            The periods
	 */
	private void processFile(ArrayList<PeriodBean> periods) {

		boolean hasPeriodGreaterThan100 = false;
		for (int i = 0; i < periods.size(); i++) {

			PeriodBean period = periods.get(i);

			if (period.getDuration() > 99) {
				hasPeriodGreaterThan100 = true;
			}
		}

		int totalDuration = 0;
		for (int i = 0; i < periods.size(); i++) {

			PeriodBean period = periods.get(i);

			totalDuration += period.getDuration();
			System.out
					.println(period.getDate()
							+ StringSupport
									.setMinimumLengthRight(
											" ("
													+ (period.getStart()
															.length() == 4 ? " "
															: "")
													+ period.getStart()
													+ " -> "
													+ (period.getEnd().length() == 4 ? " "
															: "")
													+ period.getEnd() + ")", 17)
							+ " = "
							+ (hasPeriodGreaterThan100 ? (period.getDuration() > 99 ? period
									.getDuration()
									: (period.getDuration() < 10 ? "  "
											+ period.getDuration() : " "
											+ period.getDuration()))
									: period.getDuration()) + "min ["
							+ period.getComment() + "]");
		}

		int hours = totalDuration / 60;
		int minutes = totalDuration - (hours * 60);
		System.out.println("Durao total: " + totalDuration + "min = " + hours
				+ ":" + (minutes < 10 ? "0" : "") + minutes + "h");
	}

	/**
	 * Retrieve the periods from file.
	 * 
	 * @param fileName
	 *            The filename.
	 * @return The periods
	 */
	private static ArrayList<PeriodBean> retrievePeriods(String fileName) {

		ArrayList<PeriodBean> periods = new ArrayList<PeriodBean>();

		String lines[] = FileSupport.readAllLines(fileName);

		for (int i = 0; i < lines.length / 2; i++) {

			PeriodBean period = new PeriodBean();

			StringTokenizer tokenizer = new StringTokenizer(lines[i * 2], ",");

			String action = tokenizer.nextToken().trim();

			if (!PeriodBean.START.equals(action)) {
				throw new RuntimeException("Invalid action on file");
			}
			String date = tokenizer.nextToken().trim();

			period.setDate(convertDate(date));

			String hour = tokenizer.nextToken().trim();
			StringTokenizer startHourTokenizer = new StringTokenizer(hour, ":");

			String hourStart = startHourTokenizer.nextToken().trim();
			String minuteStart = startHourTokenizer.nextToken().trim();

			period.setStart(hourStart + ":" + minuteStart);

			String commentStart = tokenizer.nextToken().trim();

			tokenizer = new StringTokenizer(lines[i * 2 + 1], ",");

			action = tokenizer.nextToken().trim();
			if (!PeriodBean.END.equals(action)) {
				throw new RuntimeException("Invalid action on file");
			}

			date = tokenizer.nextToken().trim();

			hour = tokenizer.nextToken().trim();
			StringTokenizer endHourTokenizer = new StringTokenizer(hour, ":");

			String hourEnd = endHourTokenizer.nextToken().trim();
			String minuteEnd = endHourTokenizer.nextToken().trim();

			period.setEnd(hourEnd + ":" + minuteEnd);

			int hourStartNumber = Integer.parseInt(hourStart);
			int minuteStartNumber = Integer.parseInt(minuteStart);
			int hourEndNumber = Integer.parseInt(hourEnd);
			int minuteEndNumber = Integer.parseInt(minuteEnd);

			int hoursDifference = hourEndNumber - hourStartNumber;
			int minutesDiference = minuteEndNumber - minuteStartNumber;

			int totalMinutes = hoursDifference * 60 + minutesDiference;

			period.setDuration(totalMinutes);

			String commentEnd = tokenizer.nextToken().trim();

			period.setComment(commentStart + " -> " + commentEnd);

			periods.add(period);
		}
		return periods;
	}

	private static String getDayOfTheWeek() {

		Calendar calendar = Calendar.getInstance();

		if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
			return "Sun";
		}
		if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY) {
			return "Mon";
		}
		if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.TUESDAY) {
			return "Tue";
		}
		if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.WEDNESDAY) {
			return "Wed";
		}
		if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.THURSDAY) {
			return "Thu";
		}
		if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY) {
			return "Fri";
		}
		if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SATURDAY) {
			return "Sat";
		}

		return null;
	}

	/**
	 * Print the date and time.
	 */
	private static void printDateTime() {

		Calendar calendar = Calendar.getInstance();

		System.out.print((calendar.get(Calendar.MONTH) + 1) + "/"
				+ calendar.get(Calendar.DAY_OF_MONTH) + "/"
				+ calendar.get(Calendar.YEAR) + ","
				+ calendar.get(Calendar.HOUR_OF_DAY) + ":"
				+ calendar.get(Calendar.MINUTE) + ":"
				+ calendar.get(Calendar.SECOND) + ".00,");
	}

	/**
	 * Analyze the timesheet file.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		Timesheet timesheet = new Timesheet();

		if (args.length == 0) {
			return;
		}

		String parameter = args[0];

		if ("i".equals(parameter.trim()) || "f".equals(parameter.trim())) {
			Calendar calendar = Calendar.getInstance();

			if (calendar.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) {
				System.out.print("Sun");
			}

			if ("i".equals(parameter.trim())) {

				System.out.print("Start,");
			} else {
				System.out.print("End,");
			}
			System.out.print(getDayOfTheWeek() + " ");
			printDateTime();

			for (int i = 1; i < args.length; i++) {
				System.out.print(args[i] + " ");
			}

			System.out.println("      ");

			return;
		}

		if ("c".equals(parameter.trim())) {
			Calendar calendar = Calendar.getInstance();

			System.out.print(calendar.get(Calendar.YEAR) + "-"
					+ (calendar.get(Calendar.MONTH) + 1) + "-"
					+ calendar.get(Calendar.DAY_OF_MONTH) + "_"
					+ calendar.get(Calendar.HOUR_OF_DAY) + "h"
					+ calendar.get(Calendar.MINUTE) + "m");

			for (int i = 1; i < args.length; i++) {
				System.out.print(args[i] + " ");
			}

			return;
		}

		ArrayList<PeriodBean> periods = retrievePeriods(parameter);
		timesheet.processFile(periods);
		generateChart(periods);
	}
}
