/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.util;

import java.awt.Component;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.text.DateFormat;
import java.util.Date;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 * Support the handling of simple exceptions. 
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class ExceptionSupport {

	// The default is to use swing messages.
	private static boolean useSwingMessages = true;

	/**
	 * Write a date stamp on the output
	 */
	private static void writeDate() {

		Err.print(DateFormat.getDateInstance().format(new Date()) + " - "
				+ DateFormat.getTimeInstance().format(new Date()) + " [ERROR] ");
	}

	/**
	 * Show a message dialog and generate the entrace in the
	 * execution error message.
	 * @param mainComponent The main component. If null than the dialog is screen centered.
	 * @param message The exception message.
	 */
	public static void handleException(Component mainComponent, String message) {

		writeDate();
		Err.println(message);

		if (useSwingMessages) {
			Component relativeComponent = (mainComponent == null) ? relativeComponent = SwingUtil
					.retrieveCenteredTemporaryFrame() : mainComponent;
			JOptionPane.showMessageDialog(relativeComponent, message,
					InternalResource.get(InternalResource.EXCEPTION_OCCURRED),
					JOptionPane.ERROR_MESSAGE);

			if (relativeComponent != mainComponent) {
				((JFrame) relativeComponent).dispose();
			}
		}
	}

	/**
	 * Show a message dialog and generate the entrace in the
	 * execution error message.
	 * @param message The exception message.
	 */
	public static void handleException(String message) {

		handleException(null, message);
	}

	/**
	 * Show a message dialog and generate the entrace in the
	 * execution error file.
	 * @param exception The exception.
	 */
	public static void handleException(Throwable exception) {

		writeDate();

		String message = throwableToString(exception);
		Err.println(message);

		if (useSwingMessages) {
			JFrame tempFrame = SwingUtil.retrieveCenteredTemporaryFrame();
			JOptionPane.showMessageDialog(tempFrame, message,
					InternalResource.get(InternalResource.EXCEPTION_OCCURRED),
					JOptionPane.ERROR_MESSAGE);
			tempFrame.dispose();
		}
	}

	/**
	 * Transform an exception into a string.
	 * @param throwable The exception
	 * @return The exception in string format.
	 */
	public static String throwableToString(Throwable throwable) {

		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		PrintStream printStream = new PrintStream(outputStream);
		throwable.printStackTrace(printStream);

		return outputStream.toString();
	}

	/**
	 * Turn the swing alert windows on of off 
	 * @param useSwingMessages the useSwingMessages to set
	 */
	public static void setUseSwingMessages(boolean useSwingMessages) {

		ExceptionSupport.useSwingMessages = useSwingMessages;
	}

}
