/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.util;

import java.awt.event.KeyEvent;

import javax.swing.Action;
import javax.swing.Icon;

/**
 * Provide basic actions to swing applications.
 * 
 * @author Carlos Fernando Bella Cruz - <a
 *         href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class SimpleActionProvider {

	/**
	 * Convert the first character of a string into a key event.
	 * 
	 * @param character
	 *            The string containing the character to be converted to key
	 *            event.
	 * @return The key event.
	 */
	public static int getKeyEvent(String character) {

		switch (Character.toUpperCase(character.charAt(0))) {
		case 'A': {
			return KeyEvent.VK_A;
		}
		case 'B': {
			return KeyEvent.VK_B;
		}
		case 'C': {
			return KeyEvent.VK_C;
		}
		case 'D': {
			return KeyEvent.VK_D;
		}
		case 'E': {
			return KeyEvent.VK_E;
		}
		case 'F': {
			return KeyEvent.VK_F;
		}
		case 'G': {
			return KeyEvent.VK_G;
		}
		case 'H': {
			return KeyEvent.VK_H;
		}
		case 'I': {
			return KeyEvent.VK_I;
		}
		case 'J': {
			return KeyEvent.VK_J;
		}
		case 'K': {
			return KeyEvent.VK_K;
		}
		case 'L': {
			return KeyEvent.VK_L;
		}
		case 'M': {
			return KeyEvent.VK_M;
		}
		case 'N': {
			return KeyEvent.VK_N;
		}
		case 'O': {
			return KeyEvent.VK_O;
		}
		case 'P': {
			return KeyEvent.VK_P;
		}
		case 'Q': {
			return KeyEvent.VK_Q;
		}
		case 'R': {
			return KeyEvent.VK_R;
		}
		case 'S': {
			return KeyEvent.VK_S;
		}
		case 'T': {
			return KeyEvent.VK_T;
		}
		case 'U': {
			return KeyEvent.VK_U;
		}
		case 'V': {
			return KeyEvent.VK_V;
		}
		case 'W': {
			return KeyEvent.VK_W;
		}
		case 'X': {
			return KeyEvent.VK_X;
		}
		case 'Y': {
			return KeyEvent.VK_Y;
		}
		case 'Z': {
			return KeyEvent.VK_Z;
		}
		}

		return 0;
	}

	/**
	 * Retrieve the action for a specific menu item
	 * 
	 * @param imageIdentification
	 *            The image
	 * @param label
	 *            The label
	 * @param mnemonicKey
	 *            The mnemonic
	 * @param shortDescription
	 *            A short description
	 * @return the decorator
	 */
	public static SimpleActionDecorator getSimpleAction(
			String imageIdentification, String label, int mnemonicKey,
			String shortDescription) {

		return getSimpleAction(imageIdentification, label, mnemonicKey,
				shortDescription, null);
	}

	/**
	 * Retrieve the action for a specific menu item
	 * 
	 * @param imageIdentification
	 *            The image
	 * @param label
	 *            The label
	 * @param mnemonicKey
	 *            The mnemonic
	 * @param shortDescription
	 *            A short description
	 * @param observer
	 *            An optional observer
	 * @return the decorator
	 */
	public static SimpleActionDecorator getSimpleAction(
			String imageIdentification, String label, int mnemonicKey,
			String shortDescription, SimpleActionObserver observer) {

		/**
		 * Inner class that represents the specific Action
		 * 
		 * @author Carlos Fernando Bella Cruz - <a
		 *         href="mailto:pessoal@carlosbcruz.com"
		 *         >pessoal@carlosbcruz.com</a>
		 */
		class SimpleAction extends SimpleActionDecorator {

			private static final long serialVersionUID = 1L;

			SimpleAction(String name, Icon icon) {

				super(name, icon);
			}
		}

		SimpleActionDecorator simpleAction = new SimpleAction(label,
				SwingUtil.loadImage(imageIdentification));

		simpleAction.putValue(Action.MNEMONIC_KEY, new Integer(mnemonicKey));

		simpleAction.putValue(Action.SHORT_DESCRIPTION, shortDescription);

		if (observer != null) {
			simpleAction.addObserver(observer);
		}

		return simpleAction;
	}
}
